"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getOffetParent(element) {
    var parent = element.parentNode;
    return parent === window.document.body || !parent ? window.document.documentElement : parent;
}
exports.getOffetParent = getOffetParent;
function getBoundingClientRect(element) {
    if (element) {
        var rect = element.getBoundingClientRect();
        var isIE = navigator.userAgent.indexOf("MSIE") !== -1;
        var rectTop = isIE && element.tagName === 'HTML' ? -element.scrollTop : rect.top;
        return {
            left: rect.left,
            top: rectTop,
            right: rect.right,
            bottom: rect.bottom,
            width: rect.right - rect.left,
            height: rect.bottom - rectTop
        };
    }
    return null;
}
exports.getBoundingClientRect = getBoundingClientRect;
/**
 * Check if the given element is fixed or is inside a fixed parent
 * @function
 * @ignore
 * @argument {Element} element
 * @argument {Element} customContainer
 * @returns {Boolean} answer to "isFixed?"
 */
function isFixed(element) {
    if (element === window.document.body) {
        return false;
    }
    if (getStyleComputedProperty(element, 'position') === 'fixed') {
        return true;
    }
    return element.parentElement ? isFixed(element.parentElement) : !!element;
}
exports.isFixed = isFixed;
/**
 * Returns the scrolling parent of the given element
 * @function
 * @ignore
 * @argument {Element} element
 * @returns {Element} offset parent
 */
function getScrollParent(element) {
    if (!element) {
        return null;
    }
    var parent = element.parentNode;
    if (!parent) {
        return element;
    }
    if (parent === window.document) {
        // Firefox puts the scrollTOp value on `documentElement` instead of `body`, we then check which of them is
        // greater than 0 and return the proper element
        if (window.document.body.scrollTop) {
            return window.document.body;
        }
        else {
            return window.document.documentElement;
        }
    }
    // Firefox want us to check `-x` and `-y` variations as well
    if (['scroll', 'auto'].indexOf(getStyleComputedProperty(parent, 'overflow')) !== -1 ||
        ['scroll', 'auto'].indexOf(getStyleComputedProperty(parent, 'overflow-x')) !== -1 ||
        ['scroll', 'auto'].indexOf(getStyleComputedProperty(parent, 'overflow-y')) !== -1) {
        // If the detected scrollParent is body, we perform an additional check on its parentNode
        // in this way we'll get body if the browser is Chrome-ish, or documentElement otherwise
        // fixes issue #65
        return parent;
    }
    return getScrollParent(element.parentElement);
}
exports.getScrollParent = getScrollParent;
/**
 * Get CSS computed property of the given element
 * @function
 * @ignore
 * @argument {Eement} element
 * @argument {String} property
 */
function getStyleComputedProperty(element, property) {
    // NOTE: 1 DOM access here
    var css = window.getComputedStyle(element, null);
    return css[property];
}
exports.getStyleComputedProperty = getStyleComputedProperty;
/**
 * Given an element and one of its parents, return the offset
 * @function
 * @ignore
 * @param {HTMLElement} element
 * @param {HTMLElement} parent
 * @param {boolean} isFixed
 * @return {Object} rect
 */
function getOffsetRectRelativeToCustomParent(element, parent, isFixed) {
    var elementRect = getBoundingClientRect(element);
    var parentRect = getBoundingClientRect(parent);
    if (elementRect != null && parentRect != null) {
        if (isFixed) {
            var scrollParent = getScrollParent(parent);
            if (scrollParent) {
                parentRect.top += scrollParent.scrollTop;
                parentRect.bottom += scrollParent.scrollTop;
                parentRect.left += scrollParent.scrollLeft;
                parentRect.right += scrollParent.scrollLeft;
            }
        }
        return {
            top: elementRect.top - parentRect.top,
            left: elementRect.left - parentRect.left,
            bottom: (elementRect.top - parentRect.top) + elementRect.height,
            right: (elementRect.left - parentRect.left) + elementRect.width,
            width: elementRect.width,
            height: elementRect.height
        };
    }
    return null;
}
exports.getOffsetRectRelativeToCustomParent = getOffsetRectRelativeToCustomParent;
/**
 * Get the outer sizes of the given element (offset size + margins)
 * @function
 * @ignore
 * @argument {Element} element
 * @returns {Object} object containing width and height properties
 */
function getOuterSizes(element) {
    // NOTE: 1 DOM access here
    var _display = element.style.display, _visibility = element.style.visibility;
    element.style.display = 'block';
    element.style.visibility = 'hidden';
    // const calcWidthToForceRepaint = element.offsetWidth;
    // original method
    var styles = window.getComputedStyle(element);
    var result = {};
    if (styles) {
        var x = parseFloat(styles.marginTop || '') + parseFloat(styles.marginBottom || '');
        var y = parseFloat(styles.marginLeft || '') + parseFloat(styles.marginRight || '');
        result = { width: element.offsetWidth + y, height: element.offsetHeight + x };
    }
    // reset element styles
    element.style.display = _display;
    element.style.visibility = _visibility;
    return result;
}
exports.getOuterSizes = getOuterSizes;
